#!/bin/nsh
# vim:syntax=sh
#******************************************************************************
# File Name:   
#
# Disclaimer:
# ==========
#   THIS SOFTWARE IS PROVIDED BY BMC Software, INC. "AS IS" AND ANY EXPRESS
#   OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
#   WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED.  IN NO EVENT SHALL BMC, INC. BE LIABLE FOR ANY
#   DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
#   DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
#   OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#   HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
#   LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
#   OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
# Description:
# ===========
# ARS Staging Directory Cleanup
#
# Parameters:
# ==========
#
#
# Exit Codes:
# ========== 
#
# 
# Revision History:
# ================
# Version  Date         By                 Description
# -------+------------+------------------+-------------------------------------
#  v1.0    14-NOV-2013	Bill Robinson		Initial Creation
#  v1.1    15-NOV-2013  Mike Jones          Add check to ensure staging area ends in ar_stage and added "-maxdepth 2" to remove the errors from the run log
#  v1.2    15-NOV-2013  Mike Jones			Changed find lines to exclude the depotfiles/.rep top level folder 
#  v1.3    15-NOV-2013  Mike Jones          Changed find lines to exclude all depotfiles as well as the .rep folder because depotfiles are copied with file creation date and could be cleaned up whilst in use for deployment
#  v1.4    18-NOV-2013  Mike Jones          Enhanced depotfiles clean so that the associated deoptfile is removed if the .rep/sub-folder is cleaned up
#
#******************************************************************************
 
#set -x			# Un-comment this to get traces of every command
 
###################
# GLOBAL VARIABLES
###################
 
###############
# When Running a script under a solaris appserver, uncomment the following section
#
#if [[ "$(uname -s)" = "SunOS" ]]
#	then
#	if [[ -z "$BLCLI_FIRST_TIME" ]]
#		then
#		LD_LIBRARY_PATH=`cat /usr/lib/rsc/HOME`/lib
#		export LD_LIBRARY_PATH
#		BLCLI_FIRST_TIME=false
#		export BLCLI_FIRST_TIME
#		exec nsh $0 "$@"
#	fi
#fi
###############
 
#####
# Rather than using a simple True/False for debug, setting a debug level
# allows us to switch on varying levels of debug, depending on what we're
# trying to achieve.
# This can be used just like the regular DEBUG flag by choosing just to use
# 0 and 1, but optionally, we can have further/different debug levels and
# hence further print_debug1 commands (see below).
# The idea here is to use levels 1-5 for debug and 0 for none.
DEBUGLEVEL=0
 
###################
# PRINT FUNCTIONS
###################
 
print_info()
{
	print "[$(date)] [INFO] ${@}"
}
 
print_error()
{
	print -u2 "[$(date)] ${COL_RED}[ERROR]${COL_RESET} ${@} ${COL_RESET}"
	exit 1
}
 
print_warn()
{
	print "${COL_YELLOW} [$(date)] ${COL_LYELLOW}[WARN]${COL_RESET} ${@} ${COL_RESET}"
	return 0
}
 
print_debug1()
{
	[[ "${DEBUGLEVEL}" -ge "1" ]] && print "[$(date)] ${COL_LBLUE}[DEBUG1]${COL_RESET} $@"
	return 0
}

print_debug2()
{
	[[ "${DEBUGLEVEL}" -ge "2" ]] && print "[$(date)] ${COL_CYAN}[DEBUG2]${COL_RESET} $@"
	return 0
}
 
print_debug3()
{
	[[ "${DEBUGLEVEL}" -ge "3" ]] && print "[$(date)] ${COL_LCYAN}[DEBUG3]${COL_RESET} $@"
	return 0
}
 
print_debug4()
{
	[[ "${DEBUGLEVEL}" -ge "4" ]] && print "[$(date)] ${COL_PURPLE}[DEBUG4]${COL_RESET} $@"
	return 0
}
 
print_debug5()
{
	[[ "${DEBUGLEVEL}" -ge "5" ]] && print "[$(date)] ${COL_LPURPLE}[DEBUG5]${COL_RESET} $@"
	return 0
}

print_variable()
{
	#[[ "${DEBUGLEVEL}" -ge "3" ]] && print "[$(date)] [VARIABLE] ${1}: `eval printf '%q' \\$"${1}"`"
	[[ "${DEBUGLEVEL}" -ge "3" ]] && print "[$(date)] ${COL_GREEN}[VARIABLE]${COL_RESET} ${1}: ${(P)${1}//[[:cntrl:]]/ }"
	return 0
}

print_blexec()
{
	[[ "${DEBUGLEVEL}" -ge "3" ]] && print "[$(date)] ${COL_LRED}[BLCLI_EXECUTE]${COL_RESET} $@"
    return 0
}

print_blres()
{
	[[ "${DEBUGLEVEL}" -ge "3" ]] && print "${COL_GREEN} [$(date)] ${COL_LGREEN}[BLCLI_RESULT]${COL_RESET} ${@} ${COL_RESET}"
    return 0
}
###################
# BLCLI FUNCTIONS
###################
 
#####
# Establishes a connection to BladeLogic in preparation for using the 
# BLCLI Performance Commands.
openBLConnection()
{
	print_debug2 "Entering function: ${0}..."
	if [[ -n "${CLI_JVM_OPTS:-}" ]]
		then
		for cliOpt in ${CLI_JVM_OPTS}
			do
			print_debug1 "blcli_setjvmoption ${cliOpt}"
			blcli_setjvmoption "${cliOpt}"
		done
	fi	
	# CLI_INTERACTIVE gets set to false when a script runs from 
	# within BladeLogic.  So when it's NOT false, we must be running
	# from the command line and hence need to take a few extra steps
	# before trying to establish a BLCLI connection.
	# Note that we still need to have cached session credentials for
	# this to work!
	if [[ "${CLI_INTERACTIVE}" != "false" ]]
	then
		checkBlCreds
		blcli_setoption authType BLSSO
		blcli_setoption serviceProfileName ${blProfile}
		blcli_setoption roleName "${blRole}"
	fi
 
	print_info "Opening connection to BladeLogic"
	blcli_connect 
	print_debug2 "Exiting function: ${0}..."
}
 
closeBLConnection()
{
	print_info "Closing connection to BladeLogic"
	blcli_destroy
}

checkBlCreds()
{
	print_debug2 "Entering function: ${0}..."
	blcred cred -test -profile ${blProfile}
	if [[ $? -eq 1 ]]
		then
		print_error "Please establish BladeLogic SSO credentials by running \"blcred cred -acquire -profile ${blProfile}\"..."
	fi
	print_debug2 "Exiting function: ${0}..."
}
#####
# Simple function to execute a blcli command using the performance commands
# Output is stored in the environment variable RESPONSE
# Arguments:
#   $BLCLICMD must be set prior to calling this function!
# Returns:
#   0   Success
# 999   No BLCLI supplied
# Other [As returned by BLCLI call]
runBlcliCmd()
{
	print_debug2 "Entering function: ${0}..."
	local varName="${1}" 
	local errOnFail="${2}" 
	local silent="-Dcom.bladelogic.cli.execute.quietmode.enabled=true"
	local RETCODE
	local tmpRESULT
	local RESULT
	
	if [[ -n ${BLCLICMD[@]} ]]
	then
		print_blexec "blcli_execute ${BLCLICMD[@]}"

		if [[ "${CLI_JVM_OPTS%*${silent}*}" != "${CLI_JVM_OPTS}" ]]
			then
			blcli_execute "${BLCLICMD[@]}"
		else
			blcli_execute "${BLCLICMD[@]}" > ${BLCLIOUT} 2> ${BLCLIERR}
		fi
		RETCODE=$?
 		if [[ $RETCODE -eq 0 ]] 
			then
			blcli_storeenv tmpRESULT 
			# storeenv exports the variable, we want to keep it local
			RESULT="${tmpRESULT}"
			unset tmpRESULT
			print_blres "${RESULT//[[:cntrl:]]/ }"
			# set RESULT to the variable we passed in
            if [[ "${varName}x" != "x" ]]
              	then
 				eval ${varName}="\"${RESULT}\""
			fi
			if [[ "${errOnFail}x" != "x" ]]
				then
					blcliOut="pass"
			fi
			return $RETCODE
		else
			if [[ "${errOnFail}x" = "x" ]] 
				then
				if [[ ${blVersion} -ge 82 ]]
					then
					RESULT="blcli failed..."
				else
					RESULT="$(cat ${BLCLIERR} | cut -f2- -d:)"
				fi
				print_error "${RESULT}"
			else
				blcliOut="fail"
			fi
		fi
	else
		print_error "No BLCLI command supplied!"
		return 999
	fi
	print_debug2 "Exiting function: ${0}..."
}


###################
# UTILITY FUNCTIONS
###################
 
#####
# Generic Usage command
# TO-DO: Customise on a "per script" basis
#
# Arguments:				
#   $1   Required exit code - defaults to 1.
#
usage()
{
	if [[ "${1}" = "" ]]
	then
		USAGE_CODE=1
	else
		USAGE_CODE=${1}
	fi
 
	echo "Usage: $0 -d <level> --P <blProfile> -R <blRole>"
	echo "-d	<debug level>		Debug Log output, level 0-5.  Should be first option (int)"
	echo "-R	<blRole>         	BladeLogic RBAC Role to authenticate as"
	echo "-P	<blProfile>   		BladeLogic Authentication profile to authenticate as"
	echo "-X	<jvm opts>		Options for the blcli jvm"
	echo "-r	<repeaterName>	name of specific ars"
	echo "-t	<retentionDays>	number of days to keep files, delete anything older"
	echo "-s 	<dryRun>	true|false - just print what files would be deleted"
	echo ""

	exit $USAGE_CODE
}

#####
# In various situations we need to pipe the output of certain commands to
# NULL.  Since this differs depending on the system on which the script is
# being run, we set it dynamically here.
setNull()
{
	# By piping the output of uname into grep, we don't rely on hard-coding
	# 'WindowsNT' as the system name, just in case Misrosoft change it in
	# the future.  Hopefully, whatever they use will still modify with 
	# 'Windows'...!
	uname -s | grep -q "^Windows"
	if [[ $? -eq 0 ]]
	then
		print_debug5 "Setting NULL for a Windows platform"
		NULL="NUL"
	else
		print_debug5 "Setting NULL for a UNIX platform"
		NULL="/dev/null"
	fi
}

#####
# Temp File functions
initTmpFile()
{
	! [[ -d "${TMPDIR}" ]] && mkdir -p "${TMPDIR}"
	BLCLIOUT="${TMPDIR}/blcli.out"
	BLCLIERR="${TMPDIR}/blcli.err"
}

removeTmpFile()
{
	[[ -d "${TMPDIR}" ]] && [[ "${TMPDIR}" != "/" ]] && rm -rf "${TMPDIR}"
}
 
#####
# This function initialises the environment and MUST be called at the modify
# of the main code!
initScript()
{
	setNull
	[[ ${#@} -eq 0 ]] && usage
	processParameters "$@"
	if [[ "${CLI_INTERACTIVE}" != "false" ]]
		then
		[[ "${blProfile}" = "NONE" ]] && print_error "blProfile (-P} required..."
		[[ "${blRole}" = "NONE" ]] && print_error "blRole (-R) required..."
	fi
	initTmpFile
	openBLConnection
	checkInput
}
 
#####
# This function closes down, disconnects and generally cleans up the environment
# at the end of execution and should be called at the end of the main code.
cleanupEnv()
{
	if [[ "${cleanupFiles}" = "true" ]]
		then
		print_debug1 "Cleaning up temporary files in ${TMPDIR}..."
		removeTmpFile
	else
		print_warn "Not cleaning up temporary files in ${TMPDIR}..."
	fi
	closeBLConnection

}


####################################################################
# MAIN CODE
####################################################################

#####
# All parameters should be handled in this function.
# TO-DO: Add in custom parameters for each script
processParameters()
{
	while getopts d:R:P:X:r:s:t: Option
	do
		case "${Option}" in
			d) DEBUGLEVEL="${OPTARG}"
			;;
			P) blProfile="${OPTARG}"
			   print_variable "blProfile"
			;;
			R) blRole="${OPTARG}"
			   print_variable "blRole"
			;;
			X) CLI_JVM_OPTS="${OPTARG}${CLI_JVM_OPTS:+ $CLI_JVM_OPTS}"
			   print_variable "CLI_JVM_OPTS"
			;;
			r) arsName="${OPTARG}" 
				print_variable "arsName"
			;;
			s) dryRun="${OPTARG}"
				print_variable "dryRun"
			;;
			t) retentionDays="${OPTARG}"
				print_variable "retentionDays"
			;;
			*) usage
			;;
		esac
	done
 
	[[ "${DEBUGLEVEL}" -le "0" ]] && DEBUGLEVEL=0
	[[ "${DEBUGLEVEL}" -gt "5" ]] && DEBUGLEVEL=5
	[[ "${DEBUGLEVEL}" -gt "0" ]] && print_info "Running at Debug level ${DEBUGLEVEL}"
	[[ "${DEBUGLEVEL}" -ge "6" ]] && set -x 
	[[ "${DEBUGLEVEL}" -ge "7" ]] && export PS4="$(print '[%W %*]')"

}

checkInput()
{
	print_debug2 "Entering function: ${0}..."	
	[[ ${retentionDays} -lt 4 ]] && print_error "Retention Days must be greater that 3..."

	print_debug2 "Exiting function: ${0}..."	
}



#####
# Custom Functions
#####
getAllARS()
{
	print_debug2 "Entering function: ${0}..."
	BLCLICMD=(RepeaterServer findAll)
	runBlcliCmd
	BLCLICMD=(Utility storeTargetObject arsServers)
	runBlcliCmd
	BLCLICMD=(Utility listLength)
	runBlcliCmd listLength

	for i in {0..$((${listLength}-1))}
			do
			BLCLICMD=(Utility setTargetObject arsServers)
			runBlcliCmd
			BLCLICMD=(Utility listItemSelect ${i})
			runBlcliCmd
			BLCLICMD=(Utility setTargetObject)
			runBlcliCmd
			BLCLICMD=(RepeaterServer isEnhanced)
			runBlcliCmd isARS
			if [[ "${isARS}" = "true" ]]
				then
				BLCLICMD=(RepeaterServer getName)
				runBlcliCmd arsName
				arsList=("${arsName}" ${arsList})
			fi
	done
	print_debug2 "Exiting function: ${0}..."
}

getARSStagingDir()
{
	print_debug2 "Entering function: ${0}..."
	local arsServer="${1}" && print_variable "arsServer"
	BLCLICMD=(RepeaterServer findByName "${arsServer}")
	runBlcliCmd
	BLCLICMD=(RepeaterServer getStagingDirectory)
	runBlcliCmd stagingDir
	
	print_debug2 "Exiting function: ${0}..."
}

main()
{
	print_debug2 "Entering function: ${0}..."
	typeset -a arsList
    if [[ "${arsName}" = "NONE" ]]
		then
		getAllARS
	else
		arsList=(${arsName})
	fi
	
	for ars in ${arsList}
		do
		getARSStagingDir "${ars}"
		if [[ "${stagingDir}" != "/" && -z $(echo "${stagingDir}" | grep -v ar_stage$) ]]
			then
			print_info "Processing Repeater: ${ars}, staging directory ${stagingDir}..."
			if [[ "${dryRun}" = "true" ]]
				then
				print_info "[DRY RUN] Removing files from //${ars}${stagingDir} older that ${retentionDays}..."
				find "//${ars}${stagingDir}" -mindepth 2 -maxdepth 2 -mtime +${retentionDays} -and ! -path "*depotfiles/*" -print
				find "//${ars}${stagingDir}/depotfiles" -mindepth 2 -maxdepth 2 -mtime +${retentionDays} -print
			else
				print_info "Removing files from //${ars}${stagingDir} older that ${retentionDays}..."
				find "//${ars}${stagingDir}" -mindepth 2 -maxdepth 2 -mtime +${retentionDays} -and ! -path "*depotfiles/*" -print -exec rm -rf '{}' \;
				for item in $(find "//${ars}${stagingDir}/depotfiles" -mindepth 2 -maxdepth 2 -mtime +${retentionDays} -print);
				do
					echo $item | grep -q depotfiles || continue 
				    rm -rf $item
				    DEPOTFILE=${item##*_}
				    rm -f $item/../../$DEPOTFILE
				done  
			fi
		else
			print_warn "Staging Directory for ${ars} is ${stagingDir}, skipping..."
		fi
		
	done

	print_debug2 "Exiting function: ${0}..."
}

##########################
# Base Variables
##########################
# Colors
ESC_SEQ="\x1b["
COL_RESET=${ESC_SEQ}"39;49;00m"
COL_DGRAY=${ESC_SEQ}"1;30m"
COL_RED=${ESC_SEQ}"0;31m"
COL_LRED=${ESC_SEQ}"1;31m"
COL_GREEN=${ESC_SEQ}"0;32m"
COL_LGREEN=${ESC_SEQ}"1;32m"
COL_YELLOW=${ESC_SEQ}"0;33m"
COL_LYELLOW=${ESC_SEQ}"1;33m"
COL_BLUE=${ESC_SEQ}"0;34m"
COL_LBLUE=${ESC_SEQ}"1;34m"
COL_PURPLE=${ESC_SEQ}"0;35m"
COL_LPURPLE=${ESC_SEQ}"1;35m"
COL_CYAN=${ESC_SEQ}"0;36m"
COL_LCYAN=${ESC_SEQ}"1;36m"
# End Colors
CLI_JVM_OPTS="-Dcom.bladelogic.cli.execute.quietmode.enabled=true"
timeStamp="$(date +%Y%m%d%H%M%S)"
blVersion="$(version | grep Network | awk '{print $4}' | cut -f1,2 -d. | tr -d .)"
SCRIPT_ARGS=$@          # In case we need it later (i.e. for parameters etc)
SCRIPT_NAME="${0}"
TMPDIR="/tmp/$$"
cleanupFiles="true"
blProfile="NONE"
blRole="NONE"
arsName="NONE"
dryRun="false"
retentionDays=365
initScript "$@"
main


#####
# End of custom code
#####
 
cleanupEnv
print_info "Exiting script..."
exit 0
 
#####################################################################
# END
#####################################################################


